
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: setmetatable

local C_Club = C_Club
local C_CreatureInfo = C_CreatureInfo
local coroutine = coroutine
local ipairs = ipairs
local Mixin = Mixin
local next = next

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local L = LibStub:GetLibrary("SixSocial-1.0", true)
if not L then return end

local ClubMembers = CreateFrame("Frame")
local Enum = L.Enum
local falsy = L.falsy
local FormatUnit = L.FormatUnit
local merge = L.merge

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function ClubMember_Delete(self)
	local clubId = self[Enum.ClubMember.ClubId]
	local memberId = self[Enum.ClubMember.Id]

	ClubMembers.clubMembers[clubId][memberId] = nil
end

local function ClubMember_Initialize(self)
	local clubId = self[Enum.ClubMember.ClubId]
	local memberId = self[Enum.ClubMember.Id]

	ClubMembers.clubMembers[clubId][memberId] = self
end

local function ClubMember_Update(self, properties)
	Mixin(self, properties)
end

-------------------------------------------------------------------------------
-- Clubs
-------------------------------------------------------------------------------

function ClubMembers:Initialize()
	if (self.initialized) then return end
	self.initialized = true
	self.clubMembers = setmetatable({}, {__index = function(t, k) t[k] = {} return t[k] end })
	self:SetScript("OnEvent", self.OnEvent)
	self:RegisterEvent("CLUB_MEMBER_PRESENCE_UPDATED")
	self:RegisterEvent("CLUB_MEMBER_REMOVED")
	self:RegisterEvent("CLUB_MEMBER_ROLE_UPDATED")
	self:RegisterEvent("CLUB_MEMBER_UPDATED")
	self:RegisterEvent("CLUB_REMOVED")
end

function ClubMembers:OnEvent(event, clubId, memberId, ...)
	local member = clubId and memberId and self.clubMembers[clubId][memberId]

	if event == "CLUB_REMOVE" then
		self.clubMembers[clubId] = nil
	elseif not member then
		return
	elseif event == "CLUB_MEMBER_PRESENCE_UPDATED" then
		local status = ...

		member:Update({
			[Enum.ClubMember.Status] = L.CLUB_MEMBER_STATUS[status],
		})
	elseif event == "CLUB_MEMBER_REMOVED" then
		member:Delete()
	elseif event == "CLUB_MEMBER_ROLE_UPDATED" then
		local role = ...

		member:Update({
			[Enum.ClubMember.Rank] = L.CLUB_MEMBER_RANK[role],
			[Enum.ClubMember.RankOrder] = role,
		})
	elseif event == "CLUB_MEMBER_UPDATED" then
		member:Delete()
	end
end

function ClubMembers:CreateClubMember(clubId, memberId)
	local memberInfo = C_Club.GetMemberInfo(clubId, memberId)
	if (not memberInfo) then return end
	local classInfo = memberInfo.classID and C_CreatureInfo.GetClassInfo(memberInfo.classID) or nil
	local status = L.CLUB_MEMBER_STATUS[memberInfo.presence]

	local member = merge(L.CLUB_MEMBER, {
		[Enum.ClubMember.AccountId] = memberInfo.bnetAccountId,
		[Enum.ClubMember.ClassName] = classInfo and classInfo.classFile,
		[Enum.ClubMember.ClubId] = clubId,
		[Enum.ClubMember.ClubType] = memberInfo.clubType,
		[Enum.ClubMember.Delete] = ClubMember_Delete,
		[Enum.ClubMember.Guid] = memberInfo.guid,
		[Enum.ClubMember.Id] = memberId,
		[Enum.ClubMember.Level] = memberInfo.level,
		[Enum.ClubMember.MythicRating] = memberInfo.overallDungeonScore,
		[Enum.ClubMember.Name] = memberInfo.name,
		[Enum.ClubMember.Note] = falsy(memberInfo.memberNote),
		[Enum.ClubMember.Rank] = memberInfo.guildRank or L.CLUB_MEMBER_RANK[memberInfo.role],
		[Enum.ClubMember.RankOrder] = memberInfo.guildRankOrder or memberInfo.role,
		[Enum.ClubMember.Status] = status,
		[Enum.ClubMember.StatusOrder] = L.Enum.StatusOrder[status],
		[Enum.ClubMember.Unit] = FormatUnit(memberInfo.name),
		[Enum.ClubMember.Update] = ClubMember_Update,
		[Enum.ClubMember.Zone] = memberInfo.zone,
		[Enum.ClubMember.Initialize] = ClubMember_Initialize,
	})

	member:Initialize()

	return member
end

-------------------------------------------------------------------------------
-- Library
-------------------------------------------------------------------------------

function L:GetClubMember(clubId, memberId)
	ClubMembers:Initialize()
	return ClubMembers.clubMembers[clubId][memberId] or ClubMembers:CreateClubMember(clubId, memberId)
end

function L:IterateCachedClubMembers(clubId)
	return coroutine.wrap(function()
		if ClubMembers.initialized then
			for _, member in next, ClubMembers.clubMembers[clubId] do
				coroutine.yield(member)
			end
		end
	end)
end

function L:IterateClubMembers(clubId)
	local members = C_Club.GetClubMembers(clubId)

	return coroutine.wrap(function()
		for _, memberId in ipairs(members) do
			local member = L:GetClubMember(clubId, memberId)

			if member then
				coroutine.yield(member)
			end
		end
	end)
end
